/*
	ACBL Remastered Controls Patch
	Copyright (C) 2017, TheFloW

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <pspsdk.h>
#include <pspkernel.h>
#include <pspctrl.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <systemctrl.h>

PSP_MODULE_INFO("ACBLRemasteredControls", 0x0007, 1, 0);

#define MAKE_CALL(a, f) _sw(0x0C000000 | (((u32)(f) >> 2) & 0x03FFFFFF), a);

#define CENTER 0x80
#define DEADZONE 0x20

STMOD_HANDLER previous;

u32 getFloatValue(float val) {
	u32 val32;
	memcpy(&val32, &val, sizeof(u32));
	return val32;
}

int cameraLeft() {
	SceCtrlData pad;
	sceCtrlPeekBufferPositive(&pad, 1);

	char rx = pad.Rsrv[0];
	rx -= CENTER;

	if (rx < -DEADZONE)
		return 1;
	
	return 0;
}

int cameraRight() {
	SceCtrlData pad;
	sceCtrlPeekBufferPositive(&pad, 1);

	char rx = pad.Rsrv[0];
	rx -= CENTER;

	if (rx > DEADZONE)
		return 1;

	return 0;
}

int cameraUp() {
	SceCtrlData pad;
	sceCtrlPeekBufferPositive(&pad, 1);

	char ry = pad.Rsrv[1];
	ry -= CENTER;

	if (ry < -DEADZONE)
		return 1;

	return 0;
}

int cameraDown() {
	SceCtrlData pad;
	sceCtrlPeekBufferPositive(&pad, 1);

	char ry = pad.Rsrv[1];
	ry -= CENTER;

	if (ry > DEADZONE)
		return 1;
	
	return 0;
}

int OnModuleStart(SceModule2 *mod) {
	char *modname = mod->modname;
	u32 text_addr = mod->text_addr;

	if (strcmp(modname, "main") == 0) {
		// Patch camera in combat mode
		MAKE_CALL(text_addr + 0x0005203C, cameraLeft);
		MAKE_CALL(text_addr + 0x00052074, cameraRight);
		MAKE_CALL(text_addr + 0x000520AC, cameraDown);
		MAKE_CALL(text_addr + 0x000520E4, cameraUp);

		// Patch camera in climbing mode
		MAKE_CALL(text_addr + 0x00066104, cameraLeft);
		MAKE_CALL(text_addr + 0x00066128, cameraRight);
		MAKE_CALL(text_addr + 0x00066170, cameraDown);
		MAKE_CALL(text_addr + 0x0006614C, cameraUp);

		// Patch camera in running mode
		MAKE_CALL(text_addr + 0x00071294, cameraLeft);
		MAKE_CALL(text_addr + 0x0007133C, cameraRight);
		MAKE_CALL(text_addr + 0x000713EC, cameraDown);
		MAKE_CALL(text_addr + 0x00071478, cameraUp);

		// Running mode: Camera left speed to +1.0
		u32 left_speed = getFloatValue(1.0f);
		_sw(0x3C040000 | (left_speed << 16), text_addr + 0x000712FC);
		_sw(0x34840000 | (left_speed & 0xFFFF), text_addr + 0x00071300);

		// Running mode: Camera right speed to -1.0
		u32 right_speed = getFloatValue(-1.0f);
		_sw(0x3C040000 | (right_speed << 16), text_addr + 0x000713A4);
		_sw(0x34840000 | (right_speed & 0xFFFF), text_addr + 0x000713A8);

		// Allow left camera movement
		_sw(0, text_addr + 0x000712AC);
		_sw(0x50000007, text_addr + 0x000712F4);

		// Allow right camera movement
		_sw(0, text_addr + 0x00071354);
		_sw(0x50000007, text_addr + 0x0007139C);

		// Force L trigger value
		_sw(0, text_addr + 0x00052030);
		_sw(0x10000005, text_addr + 0x000660E4);
		_sw(0, text_addr + 0x00071E98);
		_sw(0x10000021, text_addr + 0x0007158C);

		sceKernelDcacheWritebackAll();
	}

	if (!previous)
		return 0;

	return previous(mod);
}

int module_start(SceSize args, void *argp) {
	previous = sctrlHENSetStartModuleHandler(OnModuleStart);
	return 0;
}